Simple "web 1.0" database-backed application.

- Pure HTML front-end
- Backed by Postgres database
- Application server uses the following:
    - Javalin webserver https://javalin.io/
    - JTE template engine https://jte.gg/


To run this demo:
- Set up postgres on your local machine
    - Then run `CREATE DATABASE books;` in psql
- If you want to run from the command line:
    - Install Gradle https://gradle.org/install/
    - `./gradlew run`
    - Navigate to http://localhost:7070/
- From VS Code:
    - Install the "Java Extensions Pack" extensions if you haven't already, which comes with Gradle.
    - Open the `crud-app-demo` folder in VS Code and let it load the project (will take a minute or so to download dependencies).
    - Open the Command Palette (Ctrl-Shift-P or Cmd-Shift-P) and type "Find Gradle Task" and select the option called "Gradle: Find Gradle Task"
        - This opens another input box, where you should type "run"
        - That will auto-navigate you to the Gradle view and highlight the run task. Press the play button to run the app.
    - Navigate to http://localhost:7070/


To understand the code in this app, read all the files below. Here's a suggested ordering for reading:
- `app/src/main/java/crud/Book.java` is the connection between the application server and the database.
- `app/src/main/java/crud/App.java` implements the HTTP routes available, each of which has a few parts:
    - validate the input
    - call the corresponding method on the Book class
    - display the result as HTML using one of the templates in `app/src/main/jte/`
- `app/src/main/jte/*.html` are the templates using JTE syntax. one possible reading order for these:
    - `view.html` for viewing a single book
    - `edit.html` and `delete.html` for operations on an existing book
    - `list.html` for displaying all books and the form to add a new book


And here are all the routes implemented by the application server:

- `/list` (also `/`): GET only
    - displays all books and a form to add a new book
    - see `App.listRoute` and `app/src/main/jte/list.html`
- `/view?id=NNN`: GET only
    - required query parameter `id` specifies the id of the book you want to view. 
    - displays that book's info and links to edit/delete.
- `/add`: POST only
    - destination of the "add new book" form from `/list` 
    - required form parameters: `title` and `author`
- `/edit`: GET or POST
    - if a GET request:
        - required query parameter `id` specifies the id of the book to edit
        - send back `app/src/main/jte/edit.html` which contains a form that targets a POST request to `/edit` 
    - if a POST request:
        - required form parameters: `id`, `title` and `author`. 
        - get the book with the given id, then update it's title to be the ones specified in the request
- `/delete`: GET or POST
    - similar to `/edit` except the form does not include title/author, and the POST route deletes the book

